import {Injectable} from '@angular/core';
import {HttpClient} from '@angular/common/http';
import {environment} from "../../environments/environment";
import {SLFDatabase} from "../models/SLFDatabase";
import {BehaviorSubject, filter, map, Observable} from "rxjs";
import {FamilyAnalysis} from "../models/FamilyAnalysis";

@Injectable({
  providedIn: 'root'
})
export class DataService {
  private readonly database: BehaviorSubject<SLFDatabase | undefined>;

  constructor(private readonly http: HttpClient) {
    this.database = new BehaviorSubject<SLFDatabase | undefined>(undefined);
  }

  public getDatabase(): Observable<SLFDatabase> {
    return <Observable<SLFDatabase>>this.database
      .asObservable()
      .pipe(filter(value => value !== undefined));
  }

  public listFamilyNames(): Observable<string[]> {
    if (this.database === undefined) {
      throw new Error('multipleFamilyAnalysis is not initialized');
    }
    return this.getDatabase()
      .pipe(map(database => Object.keys(database.families)));
  }

  public getFamily(name: string): Observable<FamilyAnalysis> {
    return this.getDatabase()
      .pipe(
        map(database => {
          if (name in database.families) {
            return database.families[name];
          } else {
            throw new Error(`Unknown family ${name}`);
          }
        })
      );
  }

  public loadAnalysisResults(): void {
    if (this.database.value !== undefined) {
      throw new Error('multipleFamilyAnalysis is already initialized');
    }

    this.http.get<SLFDatabase>(`${environment.apiUrl}/database`)
      .subscribe(database => this.database.next(database));
  }
}
